@@ -12,16 +12,21 @@ from .parser import Argument # noqa
 from .platform import pty_size # noqa
 from .program import Program # noqa
 from .runners import Runner, Local, Failure, Result # noqa
-from .tasks import task, ctask, call, Call, Task  # noqa
+from .tasks import task, call, Call, Task  # noqa
 
 
 def run(command, **kwargs):
     """
-    Invoke ``command`` in a subprocess and return a `.Result` object.
+    Invoke ``command`` in a local subprocess and return a `.Result` object.
 
-    This function is simply a convenience wrapper for creating an anonymous
-    `.Context` object and calling its `.Context.run` method, which lets you use
-    Invoke's powerful local command execution without requiring the rest of its
-    API.
+    See `.Runner.run` for API details.
+
+    .. note::
+        This function is a convenience wrapper around Invoke's `.Context` and
+        `.Runner` APIs.
+
+        Specifically, it creates an anonymous `.Context` instance and calls its
+        `~.Context.run` method, which in turn defaults to using a `.Local`
+        runner subclass for command execution.
     """
     return Context().run(command, **kwargs)
