@@ -214,8 +214,11 @@ class Config(DataProxy):
             'run': {
                 'warn': False,
                 'hide': None,
+                'shell': '/bin/bash',
                 'pty': False,
                 'fallback': True,
+                'env': {},
+                'replace_env': False,
                 'echo': False,
                 'encoding': None,
                 'out_stream': None,
